///<reference types = "@clustervr/cluster-script-types"/>

//動くスピード
const force = 3;

//プレイヤーを見つける距離
const maxDistance = 7;
//プレイヤーに近づいたら攻撃する距離
const minDistance = 0.6;

//攻撃力
const attackPower = 3;

//プレイヤーのほうを向かせるときに使う計算式
const rad2deg = (rad) => (rad * 180) / Math.PI;

const initProc = () => {
  $.state.initialized = true;
  $.state.tick = 0;
  $.state.logTick = 0;
  $.state.attackedFlag = false;
  $.state.findNeededFlag = false;

  $.state.getPosSentFlag = false;
  $.state.targetPos = new Vector3(0, 0, 0);

  $.state.moveVec = new Vector3(0, 0, 0);
};

const getPosProc = () => {
  //getPosArを初期化
  $.state.getPosAr = [];

  //このアイテムの位置を取得
  let position = $.getPosition();

  //近くにある他のアイテムを取得
  let items = $.getItemsNear(position, maxDistance);

  // それぞれのアイテムに位置を送るように指示（敵とかはこのメッセージを無視する）
  items.forEach((item) => {
    item.send("getPos", null);
  });

  $.state.getPosSentFlag = true;
};

const findProc = () => {
  $.state.getPosSentFlag = false;

  //1つもアイテムが見つからなかった場合は何もしない
  if ($.state.getPosAr.length == 0) {
    $.state.targetPlayerItem = null;
    return;
  }

  //初期化
  let targetDistance = Infinity;
  let targetPlayerItem = null;
  let targetPos = null;

  //このアイテムの位置を取得
  let position = $.getPosition();

  //$.state.getPosArには、getPosのメッセージに対応して
  //sendItemを送ってきたプレイヤー（についているアイテム）の情報が入っている

  // それぞれのプレイヤー（についているアイテム）との距離を計算し、最も近いものを探す
  $.state.getPosAr.forEach((item) => {
    //なお、距離はベクトルの長さの2乗で計算すると高速になる
    let distance = item[1].clone().sub(position).lengthSq();
    //より近いプレイヤーを対象にする
    if (distance < targetDistance) {
      targetDistance = distance;
      targetPlayerItem = item[0];
      targetPos = item[1];
    }
  });

  //見つかった（プレイヤーについている）アイテムの位置を記録
  $.state.targetPos = targetPos;
  $.state.targetPlayerItem = targetPlayerItem;
  $.state.findNeededFlag = false;
};

$.onUpdate((deltaTime) => {
  // 初期化処理
  if (!$.state.initialized) {
    initProc();
  }

  //経過時間を記録
  $.state.tick += deltaTime;

  if ($.state.attackedFlag) {
    //攻撃後は0.7秒間動かない
    $.state.attackedTick += deltaTime;
    if ($.state.attackedTick > 0.7) {
      $.state.attackedFlag = false;
      $.state.tick = 0;
    }
  } else {
    if ($.state.getPosSentFlag) {
      // getPosProcを実行してから0.3秒後に実際にプレイヤーの位置をチェックする
      if ($.state.tick >= 0.3) {
        $.state.tick -= 0.3;
        findProc();
      }
    } else {
      // 0.5秒ごとにgetPosProcを実行
      if ($.state.tick >= 0.5) {
        $.state.tick -= 0.5;
        getPosProc();
      }
    }

    //移動と攻撃
    moveProc(deltaTime);
  }
});


const moveProc = (deltaTime) => {
  //まだプレイヤーを見つけていない場合は何もしない
  if ($.state.targetPlayerItem == null) return;

  // 追跡中のプレイヤー（のアイテム）が落下やログアウトなどでいなくなっていた場合
  if ($.state.targetPlayerItem.exists() == false) {
    return;
  }

  //このアイテムの位置を取得
  let position = $.getPosition();

  // プレイヤーがいる方向の角度を計算してY軸回転
  let direction = $.state.targetPos.clone().sub(position);
  let angle = rad2deg(Math.atan2(direction.x, direction.z)); // atan2は弧度法での角度を返すため、度数法に変換する
  let rotation = new Quaternion().setFromEulerAngles(new Vector3(0, angle, 0));

  //Y軸を無視したベクトルを計算
  let xyDirection = new Vector2(direction.x, direction.z);
  let length = xyDirection.length();

  if(length > 0.1) {
    // 至近距離でない場合、角度を反映
    $.setRotation(rotation);
  }

  //まだ遠いか、攻撃後の一定時間は移動だけ行う
  if ($.state.findNeededFlag || length > minDistance) {
    //Y軸を無視して移動する量の計算
    $.state.moveVec = new Vector3(xyDirection.x, 0, xyDirection.y)
      .normalize()
      .multiplyScalar(force * deltaTime);
    //実際にこのアイテムの位置を動かす
    $.setPosition(position.add($.state.moveVec));
  } else {
    //攻撃後は改めて相手の位置を取得する必要あり
    $.state.findNeededFlag = true;

    //攻撃する
    $.state.attackedFlag = true;
    $.state.attackedTick = 0;
    $.state.targetPlayerItem.send("attack", {
      pos: position,
      power: attackPower,
    });

  }
};

//プレイヤーパーツから情報（位置）を受け取る
$.onReceive((messageType, arg, sender) => {
  switch (messageType) {
    case "sendItem":
      //$.state.getPosArに、送られてきたプレイヤー（についているアイテム）の情報を記録
      //注意：直接$.state.getPosAr.push(sender)と書くと、反映されない
      let ar = $.state.getPosAr;
      ar.push([sender, arg]);
      $.state.getPosAr = ar;
      break;
  }
});
