///<reference types = "@clustervr/cluster-script-types"/>
const setPosPerSec = 0.1;

function initProc() {
  $.state.isInitialized = true;
  $.state.tick = 0;
  $.state.playerFound = false;
}

$.onUpdate((deltaTime) => {
  //初期化処理
  if (!$.state.isInitialized) {
    initProc();
  }

  //プレイヤーが見つかっていない場合はプレイヤーを探す
  if (!$.state.playerFound) {
    playerFindProc(deltaTime);
  } else {
    $.state.tick += deltaTime;

    // 0.1秒ごと（初期値）にプレイヤーの位置を更新する
    if ($.state.tick < setPosPerSec) {
      // 0.1秒経過していないときは何もしない
      return;
    }
    $.state.tick -= setPosPerSec;

    //アイテムの位置をプレイヤーの位置に合わせる
    $.setPosition($.state.playerHandle.getPosition());

    //落下するか、プレイヤーがログアウトしていたら消える
    if ($.getPosition().y < -0.5 || !$.state.playerHandle.exists()) {
      $.destroy();
      return;
    }
  }
});

const playerFindProc = (deltaTime) => {
  $.state.tick += deltaTime;
  // 1秒経過しても対象のプレイヤーが見つからなかったら消える
  if ($.state.tick > 1) {
    $.destroy();
    return;
  }

  // 0.4m以内にプレイヤーがいるか？
  let near = $.getPlayersNear($.getPosition(), 0.4);
  near.forEach((player) => {
    //最初に見つかったプレイヤーを対象にする
    $.state.playerFound = true;
    $.state.playerHandle = player;
    return;
  });
};

const isNearPlayerPos = (pos) => {
  //そもそもプレイヤーが見つかっていない場合は当たらない
  if($.state.playerHandle == null) return false;

  //プレイヤーの位置を取得
  let playerPos = $.state.playerHandle.getPosition();
  if(playerPos.y > 0.3) return false; //ジャンプ中は当たらない

  //プレイヤーの位置とアイテムの位置の差を計算 ただしY軸は無視、0.7m以内なら当たったとみなす
  playerPos.y = 0;
  pos.y = 0;
  return playerPos.sub(pos).length() < 0.7;
}

const dealDamage = (damage) => {
  //まず現在のHPを取得
  let hp = $.getStateCompat("owner", "HP","integer");

  //ダメージ分だけHPを減らす
  hp -= damage;

  //HPが0以下になったら
  if(hp <= 0) {
    //マイナスにならないように0にする
    hp = 0;
    //プレイヤーを床の下へ移動させる
    $.state.playerHandle.setPosition(new Vector3(0, -2, 0));
  }
  //プレイヤーのHPを更新
  $.setStateCompat("owner","HP", hp);

  //音を鳴らすためのシグナルを送る
  $.sendSignalCompat("this", "attacked");

  if(hp <= 0) {
    //プレイヤーのHPが0になったら消える
    $.destroy();
  }
}

//敵からのメッセージを受け取る
$.onReceive((messageType, arg, sender) => {
  switch (messageType) {
    case "getPos":
      //このアイテムの位置を送る
      sender.send("sendItem", $.getPosition());
      break;
    case "attack":
      //相手の座標が近くにあるときのみダメージを与える
      if (isNearPlayerPos(arg.pos)) {
        dealDamage(arg.power);
      }
    break;
  }
});
